#-------------------------------------------------------------------------#
#
# This python script calculates the differences between numerical and 
# analytic results. It requires NetCDF output files and the Scientific 
# and numpy python modules installed. 
#
#Copyright (C) 2009 David McInerney, University of Chicago
#
#-------------------------------------------------------------------------
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program (see file "COPYING").  If not, see 
#    <http://www.gnu.org/licenses/>.
#
#-------------------------------------------------------------------------

#!/usr/bin/env python
from Scientific.IO.NetCDF import NetCDFFile
from numpy import mean, abs, size, shape, zeros, max
from numpy.linalg import norm

#file_1 = '../Output/Manuscript/Test1/28_20_2000/output.nc'
#file_2 = '../Output/Manuscript/Test2/28_20_2000/output.nc'
#file_3 = '../Output/Manuscript/Test3/28_20_2000/output.nc'
file_1 = '../Output/Manuscript/Test1/56_40_8000/output.nc'
file_2 = '../Output/Manuscript/Test2/56_40_8000/output.nc'
file_3 = '../Output/Manuscript/Test3/56_40_8000/output.nc'
#file_1 = '../Output/Manuscript/Test1/140_100_50000/output.nc'
#file_2 = '../Output/Manuscript/Test2/140_100_50000/output.nc'
#file_3 = '../Output/Manuscript/Test3/140_100_50000/output.nc'

def print_diffs(file):

   print file

   f = NetCDFFile(file)   

   x_at_zeta = f.variables['x_at_zeta'].getValue()
   y_at_zeta = f.variables['y_at_zeta'].getValue()

   x = f.variables['x'].getValue()
   y = f.variables['y'].getValue()
   time = f.variables['time'].getValue()

   JJ = size(x_at_zeta)
   KK = size(y_at_zeta)
   NN = size(time)

   zeta_num = f.variables['Numerical_zeta'].getValue()
   zeta_anal = f.variables['Analytic_zeta'].getValue()
   zeta_1 = f.variables['Analytic_zeta1'].getValue()
   zeta_2 = f.variables['Analytic_zeta2'].getValue()
   zeta_anal_at_zeta = zeta_anal[:,1:2*KK:2,1:2*JJ:2]
   zeta_1_at_zeta = zeta_1[:,1:2*KK:2,1:2*JJ:2]
   zeta_2_at_zeta = zeta_2[:,1:2*KK:2,1:2*JJ:2]
   u_num = f.variables['Numerical_U'].getValue()
   v_num = f.variables['Numerical_V'].getValue()
#   u_num_at_zeta = zeros((NN,KK,JJ),'f')
#   v_num_at_zeta = zeros((NN,KK,JJ),'f')
   u_num_at_zeta = 0.5*(u_num[:,:,0:JJ]+u_num[:,:,1:JJ+1])
   v_num_at_zeta = 0.5*(v_num[:,0:KK,:]+v_num[:,1:KK+1,:])
   u_anal = f.variables['Analytic_U'].getValue()
   u_1 = f.variables['Analytic_U1'].getValue()
   u_2 = f.variables['Analytic_U2'].getValue()
   v_anal = f.variables['Analytic_V'].getValue()
   v_1 = f.variables['Analytic_V1'].getValue()
   v_2 = f.variables['Analytic_V2'].getValue()
#   u_anal_at_zeta = zeros((NN,KK,JJ),'f')
#   v_anal_at_zeta = zeros((NN,KK,JJ),'f')
   u_anal_at_zeta = u_anal[:,1:2*KK:2,1:2*JJ:2] 
   v_anal_at_zeta = v_anal[:,1:2*KK:2,1:2*JJ:2]
#   u_1_at_zeta = zeros((NN,KK,JJ),'f')
#   v_1_at_zeta = zeros((NN,KK,JJ),'f')
   u_1_at_zeta = u_1[:,1:2*KK:2,1:2*JJ:2]
   v_1_at_zeta = v_1[:,1:2*KK:2,1:2*JJ:2]
#   u_2_at_zeta = zeros((NN,KK,JJ),'f')
#   v_2_at_zeta = zeros((NN,KK,JJ),'f')
   u_2_at_zeta = u_2[:,1:2*KK:2,1:2*JJ:2]
   v_2_at_zeta = v_2[:,1:2*KK:2,1:2*JJ:2]

   f.close()

   zeta_diff_abs_sum = 0.
   zeta_diff1_abs_sum = 0.
   zeta_anal_abs_sum = 0.
   zeta_1_abs_sum = 0.
   zeta_2_abs_sum = 0.
   zeta_num_abs_sum = 0.
   q_diff_abs_sum = 0.
   q_diff1_abs_sum = 0.
   q_anal_abs_sum = 0.
   q_1_abs_sum = 0.
   q_2_abs_sum = 0.
   q_1_abs_max = 0.
   q_2_abs_max = 0.
   q_num_abs_sum = 0.
   counter = 0
   for n in range(1,NN):
      for k in range(0,KK):
         for j in range(0,JJ):
            zeta_diff_abs_sum += abs( zeta_num[n,k,j]-zeta_anal_at_zeta[n,k,j] )
            zeta_anal_abs_sum += abs( zeta_anal_at_zeta[n,k,j] )
            zeta_diff1_abs_sum += abs( zeta_num[n,k,j]-zeta_1_at_zeta[n,k,j] )
            zeta_1_abs_sum += abs( zeta_1_at_zeta[n,k,j] )
            zeta_2_abs_sum += abs( zeta_2_at_zeta[n,k,j] )
            zeta_num_abs_sum += abs( zeta_num[n,k,j] )
            q_diff_abs_sum += norm( [ u_num_at_zeta[n,k,j]-u_anal_at_zeta[n,k,j], \
	   				v_num_at_zeta[n,k,j]-v_anal_at_zeta[n,k,j] ] )
            q_anal_abs_sum += norm( [ u_anal_at_zeta[n,k,j], v_anal_at_zeta[n,k,j] ] )
            q_diff1_abs_sum += norm( [ u_num_at_zeta[n,k,j]-u_1_at_zeta[n,k,j], \
                                        v_num_at_zeta[n,k,j]-v_1_at_zeta[n,k,j] ] )
            q_1_abs_sum += norm( [ u_1_at_zeta[n,k,j], v_1_at_zeta[n,k,j] ] )
            q_1_abs = norm( [ u_1_at_zeta[n,k,j], v_1_at_zeta[n,k,j] ] )
            q_2_abs = norm( [ u_2_at_zeta[n,k,j], v_2_at_zeta[n,k,j] ] )
            if q_1_abs > q_1_abs_max:
               q_1_abs_max = q_1_abs
            if q_2_abs > q_2_abs_max:
               q_2_abs_max = q_2_abs
            q_1_abs_sum += q_1_abs
            q_2_abs_sum += q_2_abs
            q_num_abs_sum += norm( [ u_num_at_zeta[n,k,j], v_num_at_zeta[n,k,j] ] )
            counter += 1

   print
   print "max abs zeta1:",max(abs(zeta_1_at_zeta))
   print "mean abs zeta1:",zeta_1_abs_sum/counter
   print
   print "max abs zeta 2 / max abs zeta 1:", max(abs(zeta_2_at_zeta))/max(abs(zeta_1_at_zeta))
   print "mean abs zeta2 / mean abs zeta1:", zeta_2_abs_sum/zeta_1_abs_sum
   print
   print "max abs q2 / max abs q1:", q_2_abs_max/q_1_abs_max
   print "mean abs q2 / mean abs q1:", q_2_abs_sum/q_1_abs_sum
   print
   print "mean abs zeta diff / mean abs zeta anal:", zeta_diff_abs_sum/zeta_anal_abs_sum*100.
   print "mean abs q diff / mean abs q anal:",q_diff_abs_sum/q_anal_abs_sum*100.
   print "mean abs zeta diff1 / mean abs zeta anal1:", zeta_diff1_abs_sum/zeta_1_abs_sum*100.
   print "mean abs q diff1 / mean abs q anal1:",q_diff1_abs_sum/q_1_abs_sum*100.

print "Test 1"
print_diffs(file_1)

print "Test 2"
print_diffs(file_2)

print "Test 3"
print_diffs(file_3)





